#include "uart.h"
#include "nRF51.h"
#include "nRF51_bitfields.h"
#include "board_config.h"
#include "nrf_gpio.h"
#include <stdarg.h>
#include <stdio.h>


static const uint32_t m_baudrates[UART_BAUD_TABLE_MAX_SIZE] = UART_BAUDRATE_DEVISORS_ARRAY;

void uart_init(uart_baudrate_t const baud_rate)
{

  nrf_gpio_cfg_output(BOARD_UART0_TX);
  nrf_gpio_cfg_input(BOARD_UART0_RX, NRF_GPIO_PIN_NOPULL);	
	
  NRF_UART0->PSELTXD = BOARD_UART0_TX;
  NRF_UART0->PSELRXD = BOARD_UART0_RX;

  NRF_UART0->ENABLE           = 0x04;
  NRF_UART0->BAUDRATE         = m_baudrates[baud_rate];
  NRF_UART0->CONFIG           = (UART_CONFIG_HWFC_Enabled << UART_CONFIG_HWFC_Pos); 
  NRF_UART0->TASKS_STARTTX    = 1;
  NRF_UART0->TASKS_STARTRX    = 1;
  NRF_UART0->EVENTS_RXDRDY    = 0;
}

bool hal_uart_chars_available(void)
{
  return (NRF_UART0->EVENTS_RXDRDY == 1);
}

void sendchar(uint8_t ch)
{
  uart_putchar(ch);
}

void uart_putchar(uint8_t ch)
{
  NRF_UART0->EVENTS_TXDRDY = 0;
  NRF_UART0->TXD = ch;
  while(NRF_UART0->EVENTS_TXDRDY != 1){}  // Wait for TXD data to be sent
  NRF_UART0->EVENTS_TXDRDY = 0;
}

uint8_t uart_getchar(void)
{
  while(NRF_UART0->EVENTS_RXDRDY != 1){}  // Wait for RXD data to be received
  NRF_UART0->EVENTS_RXDRDY = 0;
  return (uint8_t)NRF_UART0->RXD;
}

void uart_write_buf(uint8_t const *buf, uint32_t len)
{
  while(len--)
    uart_putchar(*buf++);
}

/**@brief Logging function, used for formated output on the UART.
 */
void uart_logf(const char *fmt, ...)
{
    uint16_t i = 0;
    static uint8_t logf_buf[150];
    va_list args;
    va_start(args, fmt);    
    i = vsnprintf((char*)logf_buf, sizeof(logf_buf) - 1, fmt, args);
    logf_buf[i] = 0x00; /* Make sure its zero terminated */
    uart_write_buf((uint8_t*)logf_buf, i);
    va_end(args);
} 


uint8_t uart_getline(uint8_t * string, uint8_t max_len)
{
    uint8_t c, k;
    c = '\0';
    for ( k = 0; k < max_len - 1 ; k++ )
    {
        c = uart_getchar();
        if (c == UART_NEWLINE_INPUT ) 
				{
					uart_putchar(c);
					break;
				}
        string[k] = c;
        uart_putchar(c);
    }
    string[k] = 0;
		return k;
} 

void simple_uart_put(uint8_t cr)
{
	uart_putchar(cr);
}

void simple_uart_config(uint8_t rts_pin_number, uint8_t txd_pin_number, uint8_t cts_pin_number, uint8_t rxd_pin_number, bool hwfc)
{
	uart_init(UART_BAUD_38K4);
}
